/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 * 
 * You can obtain a copy of the license at
 * https://jwsdp.dev.java.net/CDDLv1.0.html
 * See the License for the specific language governing
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * https://jwsdp.dev.java.net/CDDLv1.0.html  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */
 
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.util.List;

// import java content classes generated by binding compiler
import example.*;

/*
 * $Id: Main.java,v 1.1 2008/11/03 06:51:20 greggw Exp $
 *
 * Copyright 2003 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
 
public class Main {
    
    // This sample application demonstrates how to modify a collection
    // in a java content tree and marshal it back to xml data
    
    public static void main( String[] args ) {
        try {
            // create a JAXBContext capable of handling classes generated into
            // the example package
            JAXBContext jc = JAXBContext.newInstance( "example" );
            
            // create an Unmarshaller
            Unmarshaller u = jc.createUnmarshaller();
            
            // unmarshal a FooBar instance document into a tree of Java content
            // objects composed of classes from the example package.
            FooBar fb = 
                (FooBar)u.unmarshal( new FileInputStream( "example.xml" ) );

	    // read and write unnamed model group bound to Java content interface
	    FooBar.PhoneNumberOrSpeedDial numOrSD = 
					fb.getPhoneNumberOrSpeedDial();
	    System.out.println("PhoneNumberOrSpeedDial value:" + numOrSD.getContent()+
		               " isPhoneNumber:" + numOrSD.isSetPhoneNumber() + 
		               " isSpeedDial:" + numOrSD.isSetSpeedDial());
            numOrSD.setSpeedDial(22);
	    
	    // read and write model group def.bound to Java content interface
            ModelGroupChoice mgc = fb.getModelGroupChoice();
	    System.out.println("ModelGroupChoice value:" + mgc.getContent()+
		               " isBool:" + mgc.isSetBool() + 
		               " isComment:" + mgc.isSetComment() + 
		               " isValue:" + mgc.isSetValue());
	    mgc.setBool(false);
	     
            // create a Marshaller and marshal to a file
            Marshaller m = jc.createMarshaller();
            m.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE );
            m.marshal( fb, System.out );
            
        } catch( JAXBException je ) {
            je.printStackTrace();
        } catch( IOException ioe ) {
            ioe.printStackTrace();
        }
    }
}
